@@ -23,7 +23,7 @@ const char *PlatformUtils::OPENSCAD_FOLDER_NAME = "OpenSCAD";
 static std::string lookupResourcesPath()
 {
 	fs::path resourcedir(applicationpath);
-	PRINTDB("Looking up resource folder with application path '%s'", resourcedir.c_str());
+	PRINTDB("Looking up resource folder with application path '%s'", boosty::stringy(resourcedir).c_str());
 	
 #ifdef __APPLE__
 	const char *searchpath[] = {
@@ -56,12 +56,14 @@ static std::string lookupResourcesPath()
 	for (int a = 0;searchpath[a] != NULL;a++) {
 	    tmpdir = resourcedir / searchpath[a];
 	    
-	    const fs::path checkdir = tmpdir / "libraries";
-	    PRINTDB("Checking '%s'", checkdir.c_str());
+			// The resource folder is the folder which contains "color-schemes" (as well as 
+			// "examples" and "locale", and optionally "libraries" and "fonts")
+	    const fs::path checkdir = tmpdir / "color-schemes";
+	    PRINTDB("Checking '%s'", boosty::stringy(checkdir).c_str());
 
 	    if (is_directory(checkdir)) {
 		resourcedir = tmpdir;
-		PRINTDB("Found resource folder '%s'", tmpdir.c_str());
+		PRINTDB("Found resource folder '%s'", boosty::stringy(tmpdir).c_str());
 		break;
 	    }
 	}
