@@ -89,17 +89,17 @@ AbstractNode *CgaladvModule::instantiate(const Context *ctx, const ModuleInstant
 		node->newsize << 0,0,0;
 		if ( ns->type() == Value::VECTOR ) {
 			const Value::VectorType &vs = ns->toVector();
-			if ( vs.size() >= 1 ) node->newsize[0] = vs[0].toDouble();
-			if ( vs.size() >= 2 ) node->newsize[1] = vs[1].toDouble();
-			if ( vs.size() >= 3 ) node->newsize[2] = vs[2].toDouble();
+			if ( vs.size() >= 1 ) node->newsize[0] = vs[0]->toDouble();
+			if ( vs.size() >= 2 ) node->newsize[1] = vs[1]->toDouble();
+			if ( vs.size() >= 3 ) node->newsize[2] = vs[2]->toDouble();
 		}
 		ValuePtr autosize = c.lookup_variable("auto");
 		node->autosize << false, false, false;
 		if ( autosize->type() == Value::VECTOR ) {
 			const Value::VectorType &va = autosize->toVector();
-			if ( va.size() >= 1 ) node->autosize[0] = va[0].toBool();
-			if ( va.size() >= 2 ) node->autosize[1] = va[1].toBool();
-			if ( va.size() >= 3 ) node->autosize[2] = va[2].toBool();
+			if ( va.size() >= 1 ) node->autosize[0] = va[0]->toBool();
+			if ( va.size() >= 2 ) node->autosize[1] = va[1]->toBool();
+			if ( va.size() >= 3 ) node->autosize[2] = va[2]->toBool();
 		}
 		else if ( autosize->type() == Value::BOOL ) {
 			node->autosize << autosize->toBool(),autosize->toBool(),autosize->toBool();
